(function(global) {
  'use strict';

  angular.module('http-with-fallback', [])
    .factory('httpWithFallback', ['$http', '$q','DeferredWithUpdate', function($http, $q, DeferredWithUpdate) {


      // Constructor function, using $http as prototype
      function HttpWithFallback() { }
      HttpWithFallback.prototype = $http;
      HttpWithFallback.prototype.constructor = HttpWithFallback;

      var httpWithFallback = new HttpWithFallback();
         // localStorage = global.localStorage;

      /**
       * Override $http.get to catch the promise.error
       */
      httpWithFallback.get = function(url, config, getCache ) {      
      	
        // Client doesn't support local storage
       
        var works = 1;
        try{
        	if (!global.localStorage)	
        		works = 0;
        	else
        		works = 1;
        }catch(e){ works = 0; }
        
        if ( !works ) { 
          // If no fallback defined, just just $http.get
          if (!config.fallback) {
            return $http.get(url, config);
          }
          // Local storage won't be used
          config.dontStoreFallback = true;
        }

        // Delegate get to $http
        var deferred = DeferredWithUpdate.defer();//$q.defer();
        
        /*
        var limit = 1024 * 1024 * 5; // 5 MB
        var remSpace = limit - unescape(encodeURIComponent(JSON.stringify(localStorage))).length;
        console.log("remaining space => "+remSpace+" << ");*/
        
        //SET CACHED VALUE
  
        if( getCache )
        {
        	try{
		var storedResponse = localStorage.getItem(url);
		}
		catch(e){}
		
		    if (storedResponse) {
		      // Data was successfully retrieved from local storage, resolve with status 200
		      //alert(storedResponse);
		      var error = 0;
		      storedResponse = tryParseJSON(storedResponse);
			 
		      if( typeof storedResponse != 'undefined' )
		      {
		      	      if( typeof storedResponse.data != 'undefined' )
		      	      {
		      	      	      if( typeof storedResponse.data == 'string' )
		      	      	      	      storedResponse.data = {};
				      //alert(storedResponse.data.ret.players);
				      var headers = storedResponse.headers;
				      storedResponse.headers = function() { return headers; };
				      storedResponse.data.cachedCopy = 1;
				      storedResponse.data.cache      = 1;
				      deferred.resolve(storedResponse);
			      }
			      else
			      	  	error = 1;
			      
		      }
		      else
		      	      error = 1;
		      
		      //( error == 1 )
		      //	      deferred.resolve({data:{cache:1,cachedCopy:1;status:'FAILUIRE'}});
		      
		      //alert('din cache');
		      //alert('cache='+JSON.stringify(storedResponse));
		    }
	}
        
        
        
         
        
        $http.get(url, config)
        .then(
          function(response) {
              
            // Store in local storage when status === 200
           
            if (!response.config.dontStoreFallback && response.status === 200) {
            	
            	    //alert(JSON.stringify(response.data.ret.players ) );
            	
            	if( typeof response.data == 'object' ){
			var stringification = JSON.stringify({
						  data: response.data,
						  status: response.status,
						  config: response.config,
						  headers: response.headers(),
						  isFallback: true
						});
		   
			saveLocalStorage( url, stringification );
		       
              	} 
            }
            // Resolve with original response
            //alert( response );
            // alert( response  );
            deferred.resolve(response);
          },
          function(response) {
            // Try to retrieve from local storage
          
            var storedResponse = localStorage.getItem(url);
            if (storedResponse) {
              // Data was successfully retrieved from local storage, resolve with status 200
              storedResponse = JSON.parse(storedResponse);
              var headers = storedResponse.headers;
              storedResponse.headers = function() { return headers; };
              return deferred.resolve(storedResponse);
            }
            

            // Try config.fallback
            if (response.config.fallback) {
              return deferred.resolve({
                data: config.fallback,
                status: 200,
                headers: response.headers,
                config: response.config,
                isFallback: true
              });
            }

            response.data    = {};
            response.data.status  = 'FAILURE';
            response.data.ret     = {'message':tr('Server may be temporarily down')};
            response.data.message = tr('Server may be temporarily down');
      
            // Reject with original error response
            return deferred.reject(response);
          }
        );
         
        // Decorate promise with success and error functions to be compatible with the promise returned by $http.get
        var promise = deferred.promise;

        
        
        promise.success = function(fn) {
          promise.then(function(response) {
            //alert("in success");
            fn(response.data, response.status, response.headers, config, response.isFallback);
          });
          return promise;
        }; 
       

        promise.error = function(fn) {
          //alert( JSON.stringify( fn ) );
          promise.then(null, function(response) {
          		   
            fn(response.data, response.status, response.headers, config);
          });
          return promise;
        };

        return promise;
      };

      return httpWithFallback;
    }]);
    
    
    
    
}(this));